{*Adaptation of OverByteICSCliDemo by TK Boyd, see
"ver" constant for version ID

Adaptation made to create a client for a PCSensor.com TEMPerLAN*}


{* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

Author:       Franois PIETTE
Description:  Demonstration for Client program using TWSocket.
Creation:     8 december 1997
Version:      1.07
EMail:        francois.piette@overbyte.be  http://www.overbyte.be
Support:      Use the mailing list twsocket@elists.org
              Follow "support" link at http://www.overbyte.be for subscription.
Legal issues: Copyright (C) 1997-2010 by Franois PIETTE
              Rue de Grady 24, 4053 Embourg, Belgium. Fax: +32-4-365.74.56
              <francois.piette@overbyte.be>

              This software is provided 'as-is', without any express or
              implied warranty.  In no event will the author be held liable
              for any  damages arising from the use of this software.

              Permission is granted to anyone to use this software for any
              purpose, including commercial applications, and to alter it
              and redistribute it freely, subject to the following
              restrictions:

              1. The origin of this software must not be misrepresented,
                 you must not claim that you wrote the original software.
                 If you use this software in a product, an acknowledgment
                 in the product documentation would be appreciated but is
                 not required.

              2. Altered source versions must be plainly marked as such, and
                 must not be misrepresented as being the original software.

              3. This notice may not be removed or altered from any source
                 distribution.

              4. You must register this software by sending a picture postcard
                 to the author. Use a nice stamp and mention your name, street
                 address, EMail address and any comment you like to say.

Updates:
Dec 09, 1997 V1.01 Made it compatible with Delphi 1
Jul 09, 1998 V1.02 Adapted for Delphi 4
Dec 05, 1998 V1.03 Don't use TWait component
Dec 15, 2001 V1.04 Use LineMode
Jan 12, 2004 V1.05 Remove wait loop and use pure event driven code
Jul 30, 2006 V1.06 Added checkboxes to allow adding CRLF or not and to allow
                   embedded binary chars. To enter a binary char in the edit
                   box, simply enter a $ followed by thow digit hex ascii code.
                   To enter a $ sign, enter $24.
Dec 20, 2008 V1.07 Replace StrPas by a string cast. Removed an implicit
                   conversion to string by an explicit to avoid a warning.


 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *}
unit TCP001u1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, OverbyteIcsIniFiles, ExtCtrls,
  { Don't forget to add your vc32 directory to Delphi library path }
  OverbyteIcsWSocket, OverbyteIcsWndControl;

const
  ver='24 Sep 2011'; //TKB version ID
  kStartAutoRead= 'Start auto-read';// A caption changed in several places
      //and it needs to be kept consistent.
  CliDemoVersion     = 107; //ID of original application, which TKB adapted.
  CopyRight : String = 'Adaptation by TK Boyd of CliDemo (c) 1997-2010 F. Piette V1.07 ';

type
  TTCP001f1 = class(TForm)
    CliSocket: TWSocket;
    DisplayMemo: TMemo;
    Panel1: TPanel;
    Label1: TLabel;
    Label2: TLabel;
    SendButton: TButton;
    DisconnectButton: TButton;
    PortEdit: TEdit;
    ServerEdit: TEdit;
    buQuit: TButton;
    tiReadAgain: TTimer;
    buReadAgain: TButton;
    buClearMemo: TButton;
    laSheepdog: TLabel;
    Button1: TButton;
    procedure DisconnectButtonClick(Sender: TObject);
    procedure SendButtonClick(Sender: TObject);
    procedure CliSocketDataAvailable(Sender: TObject; ErrCode: Word);
    procedure CliSocketSessionConnected(Sender: TObject; ErrCode: Word);
    procedure CliSocketSessionClosed(Sender: TObject; ErrCode: Word);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormShow(Sender: TObject);
    procedure buQuitClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure buReadAgainClick(Sender: TObject);
    procedure tiReadAgainTimer(Sender: TObject);
    procedure buClearMemoClick(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    //procedure Timer1Timer(Sender: TObject); REMmed out by TKB... see not
        //in body of procedure
  private
    Buffer       : array [0..1023] of AnsiChar;
    Initialized  : Boolean;
    IniFileName  : String;
    procedure Display(Msg : String);
    procedure DisplayHex(Msg : String);
    procedure DisplayTtures(Msg : String);
    procedure ProcessCommand(Cmd : String);
    procedure SendData;
  end;

var
  TCP001f1: TTCP001f1;

implementation

uses TCP001uAbt;

{$R *.DFM}

{* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *}
procedure TTCP001f1.DisconnectButtonClick(Sender: TObject);
begin
    CliSocket.Close;
end;


{* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *}
procedure TTCP001f1.SendButtonClick(Sender: TObject);
begin
    if CliSocket.State = wsConnected then begin
        { Already connected, just send data }
        SendData;
    end
    else begin
        { Not connected yet, start connection }
        CliSocket.Proto    := 'tcp';
        CliSocket.Port     := PortEdit.Text;
        CliSocket.Addr     := ServerEdit.Text;
        CliSocket.LineMode := TRUE;
        CliSocket.LineEnd  := #255;//LineEnd can also be a short string.
        CliSocket.LineLimit:= 44;//Added by TKB... should
          //be set to the longest line you need to be able
          //to receive. If a longer line, i.e. a string
          //that has more than LineLimit characters before
          //having a LineEnd character (or sting), then
          //Things Should Happen... but unless they are subtle,
          //I am missing them! (Try setting LineLimit to, say
          //4, when the TEMPerLAN is sending 10 byte strings
          //(including the $FF at the end of the string. Program
          //ran just fine, in spite of the LineLimit setting, when
          //I tried this code with that setting. For TEMPerLAN, the
          //strings will be 4+2*s characters long, where "s" is the
          //number of sensors connected to the TEMPerLAN.
        CliSocket.Connect;
        { Connect is asynchronous (non-blocking). When the session is  }
        { connected (or fails to), we have an OnSessionConnected event }
        { This is where actual sending of data is done.                }
        SendButton.Enabled := FALSE;
    end;
end;


{* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *}

procedure TTCP001f1.SendData;
var
    Buf : String;
begin
    try
        Buf := #$BB#$82;
        CliSocket.SendStr(Buf);
    except
        on E:Exception do Display(E.ClassName + ': ' + E.Message);
    end;
    ActiveControl := SendButton;
end;


{* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *}
procedure TTCP001f1.ProcessCommand(Cmd : String);
begin
    { Here you should write your command interpreter.                       }
    { For simplicity, we just display received command !                    }
    { First remove EndOfLine marker                                         }
    if (Length(Cmd) >= Length(CliSocket.LineEnd)) and
       (Copy(Cmd, Length(Cmd) - Length(CliSocket.LineEnd) + 1,
             Length(CliSocket.LineEnd)) = String(CliSocket.LineEnd)) then
        Cmd := Copy(Cmd, 1, Length(Cmd) - Length(CliSocket.LineEnd));
    { Then display in memo                                                  }
    DisplayTtures(Cmd);
end;


{* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *}
procedure TTCP001f1.CliSocketDataAvailable(Sender: TObject; ErrCode: Word);
var
    Len : Integer;
begin
    { We use line mode, we will receive a complete line }
    Len := CliSocket.Receive(@Buffer, SizeOf(Buffer) - 1);
    if Length(Buffer) <= 0 then
        Exit;

    Buffer[Len]       := #0;    { Null terminate  }
    ProcessCommand(Buffer);
end;


{* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *}
procedure TTCP001f1.CliSocketSessionConnected(
    Sender  : TObject;
    ErrCode : Word);
begin
    SendButton.Enabled := TRUE;
    if ErrCode <> 0 then
        Display('Can''t connect, error #' + IntToStr(ErrCode))
    else begin
        DisconnectButton.Enabled := TRUE;
        buReadAgain.Enabled:=true;
        SendData;  { Send the data from edit box }
    end;
end;


{* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *}
procedure TTCP001f1.CliSocketSessionClosed(Sender: TObject; ErrCode: Word);
begin
    tiReadAgain.Enabled:=false;
    buReadAgain.Enabled:=false;
    buReadAgain.Caption:=kStartAutoRead;
    DisconnectButton.Enabled := FALSE;
    if ErrCode <> 0 then
        Display('Disconnected, error #' + IntToStr(ErrCode))
    else
        Display('Disconnected');
end;


{* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *}
procedure TTCP001f1.FormClose(Sender: TObject; var Action: TCloseAction);
var
    IniFile : TIcsIniFile;
begin
    IniFile := TIcsIniFile.Create(IniFileName);
    IniFile.WriteInteger('Window', 'Top',    Top);
    IniFile.WriteInteger('Window', 'Left',   Left);
    IniFile.WriteInteger('Window', 'Width',  Width);
    IniFile.WriteInteger('Window', 'Height', Height);
    IniFile.WriteString('Data', 'Server',  ServerEdit.Text);
    IniFile.WriteString('Data', 'Port',    PortEdit.Text);
    
    IniFile.UpdateFile;
    IniFile.Free;
end;


{* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *}
procedure TTCP001f1.FormShow(Sender: TObject);
var
    IniFile : TIcsIniFile;
begin
    if Initialized then
        Exit;
    Initialized     := TRUE;
    IniFileName     := GetIcsIniFileName;
    IniFile         := TIcsIniFile.Create(IniFileName);
    
    Top             := IniFile.ReadInteger('Window', 'Top',    Top);
    Left            := IniFile.ReadInteger('Window', 'Left',   Left);
    Width           := IniFile.ReadInteger('Window', 'Width',  Width);
    Height          := IniFile.ReadInteger('Window', 'Height', Height);

    PortEdit.Text   := IniFile.ReadString('Data', 'Port',    'telnet');
    ServerEdit.Text := IniFile.ReadString('Data', 'Server',  'localhost');


    IniFile.Free;

    DisplayMemo.Clear;
    ActiveControl := SendButton;
end;


{* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *}
{ Display a message in our display memo. Delete lines to be sure to not     
 overflow the memo which may have a limited capacity.                       }
procedure TTCP001f1.Display(Msg : String);
var
    I : Integer;
begin
    DisplayMemo.Lines.BeginUpdate;
    try
        if DisplayMemo.Lines.Count > 200 then begin
            for I := 1 to 50 do
                DisplayMemo.Lines.Delete(0);
        end;
        DisplayMemo.Lines.Add(Msg);
    finally
        DisplayMemo.Lines.EndUpdate;
{$IFNDEF VER80}
        SendMessage(DisplayMemo.Handle, EM_SCROLLCARET, 0, 0);
{$ENDIF }
    end;
end;


{* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *}
(*Note by TKB...
This Timer1 handler was in the original ICS code. I don't believe I deleted
any Timer component from the form! Believed to be a scrap
inadvertently left behind by the demo's original author.

Attempting to reach a non-existant server still times out eventually.

The code was not REMmed out in the original demo, but I have
done so for tidiness's sake...

procedure TClientForm.Timer1Timer(Sender: TObject);
begin
    if CliSocket.State = wsConnecting then
        Exit;

    if CliSocket.State <> wsConnected then
        SendButtonClick(nil)
    else
        DisconnectButtonClick(nil);
end;

(See TKB note above*)

{* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *}

{** "new" stuff from TKB below here... some derived from
 material in the demo ******************************************}

procedure TTCP001f1.DisplayHex(Msg : String);
var
    i : Integer;
    sTmp: string;
begin
  sTmp:='';//Building line to be displayed here.
  if length(Msg)>0 then begin
     for i:=1 to length(Msg) do begin
        sTmp:=sTmp+inttohex(ord(Msg[i]),2)+' ';
        end;//for...
     end;
  DisplayMemo.Lines.Add(sTmp);
end;

procedure TTCP001f1.DisplayTtures(Msg : String);
var bErr,bCountTtures:byte;
    sTmp: string;

   {subroutines of DisplayTTurees...}
   procedure CheckRecord(Msg:string; var bErrL,bCountTturesL:byte);
   //The variables do not need the "L" suffix for things to work, but
   // having them may save confusion for readers not yet fluent in matters
   // of scope.
   begin
     bErrL:=0;// if this is still zero at the end of the procedure
       //call, then no errors were found.
     bCountTturesL:=255;//a rogue value, but to give this variable
       //SOME, defined, value, in case it doesn't get one otherwise
       //displayhex(Msg); for debug only.
     if length(Msg)<5 then bErrL:=1;
     if (bErrL=0) and (Msg[1]<>#$BB) then bErrL:=2;//Check correct record 1st byte
     if (bErrL=0) and (Msg[2]<>#$82) then bErrL:=3;//Check correct record 2nd byte
     if (bErrL=0) then begin // start of block executed to
           //fetch number of readings, see if total record size right for that
           //number of readings....
           bCountTturesL:=ord(Msg[3]);
           if length(Msg)<>3+2*bCountTturesL then bErrL:=4;
          end;//Block executed to fetch number of readings, check record length
     //if bErrL still 0, then problems not found. USING the readings in the
     //  record used OUTSIDE of this subroutine.
   end;//of CheckRecord

   function CreateAnswerStringForGoodRecord(MsgL:string; bReadings:byte):string;
   var sTmp, sTmp2, sTmp3:string;
       smiTtureU,smiTtureD:smallint;//"U" units part, "D" decimal part,
          //but smiTtureU used in early stages to hold whole tture
          //while it is still expressed in 1/16ths of a Celcius degree,
          //and smiTtureD used as a "Tmp" briefly...
       bLoop:byte;
   //If TEMPerLAN had 2 sensors, reading -2.3 and 15.2, then the string
   //created should be...
   //G:02 -002.30 +015.20
   //The answer should always consist of...
   //'G:', then
   //Number of temperatures, with leading 0 if <10, then
   //The readings, each of them starting + or -, and each using
   //    three digits before the decimal point, two after it.
   //    (zero to be reported as +000.00)
   //This rigid, fixed length format, is to help programs as
   //    yet unwritten which will use the data brought this
   //    far, so far.

   //This subroutine looks fearsome, but it is just a bunch
   //    of stuff to interpret the temperatures as reported
   //    by the Dallas 18B20 chips, and to express them in
   //    the format I decided upon.

   //The readings arrive as 16 bit, 2's complement numbers.
   //    $0001,$0002,$0003.. mean 1,2,3!
   //    $FFFF is -1.... i.e. that the temperature is 1/16th
   //    of a degree Celsius below zero.

   begin
   sTmp:=inttostr(bReadings);
   if bReadings<10 then sTmp:='0'+sTmp;
   sTmp:='G:'+sTmp;

   //Use the following to insert test data, to see if various
   //   data give the correct required result.
   //One pair per reading, MSB first. Don't use more
   //   elements of MsgL that is appropriat for the number
   //   of sensors on your LAN. E.g. if you only have two
   //   sensors, rem out the MsgL[8] and MsgL[9] lines.
 {  MsgL[4]:=#$0;
   MsgL[5]:=#$1;
   MsgL[6]:=#$0;
   MsgL[7]:=#$0;
   MsgL[8]:=#$FF;
   MsgL[9]:=#$FF;
  }
   //Delphi and inttostr convert smallint type (signed 16 bit)
   //  data quite nicely... but without the leading and trailing
   //  spaced I wanted.

   //More use of real numbers was another possibility, but one
   //  I wanted to avoid.

   for bLoop:=1 to bReadings do begin
       sTmp2:=' ';//start building a number, prepended by a space
       smiTtureU:=256*ord(MsgL[2+bLoop*2])+ord(MsgL[3+bLoop*2]);
       if smiTtureU<0 then begin
          sTmp2:=' -';//start building a number, prepended by a space
          smiTtureU:=smiTtureU*-1;
          end //no ; here
         else begin
          sTmp2:=' +';//start building a number, prepended by a space
          end;//else
       //Multiply number of 16ths by 6.25 to make it number of 100ths
       smiTtureD:=smiTtureU div 4;
       smiTtureU:=smiTtureU*6+smiTtureD;
       //smiTtureU now holds tture in 100ths of deg C
       //now work out decimal part of the tture...
       smiTtureD:=smiTtureU mod 100;
       //convert smiTtureU (100ths) to degrees, shorn of fraction part
       smiTtureU:=smiTtureU div 100;
       //convert each to a string, with leading zeros as needed...
       //  adding bits to the final answer buffer as we go...
       sTmp3:=inttostr(smiTtureU);
       while length(sTmp3)<3 do sTmp3:='0'+sTmp3;
       sTmp2:=sTmp2+sTmp3+'.';

       sTmp3:=inttostr(smiTtureD);
       while length(sTmp3)<2 do sTmp3:='0'+sTmp3;
       sTmp2:=sTmp2+sTmp3;
       sTmp:=sTmp+sTmp2;
       end;//... end of loop to add bReadings ttures to output string.

   result:=sTmp;
   end;// of CreateAnswerString
   {end of subroutines of DisplayTTurees...}

begin //main block of DisplayTtures
  CheckRecord(Msg,bErr,bCountTtures);//if bErr=0 after this, all was well.
  sTmp:='There were '+inttostr(length(Msg))+' bytes in the record. ';
  if bErr=0 then sTmp:=CreateAnswerStringForGoodRecord(Msg,bCountTtures)// no ; here
            else sTmp:='B:One or more errors seen in record. '+
               'One was error ID '+inttostr(bErr);
  DisplayMemo.Lines.Add(sTmp);
  if sTmp[1]='B' then displayhex('D:'+sTmp);

end; // of subroutine DisplayTtures

procedure TTCP001f1.buQuitClick(Sender: TObject);
begin
close;
end;

procedure TTCP001f1.FormCreate(Sender: TObject);
begin
TCP001f1.caption:='TCP001: Adapted ICS demo "Client". ver: '+ver;
tiReadAgain.Interval:=2000;
tiReadAgain.Enabled:=false;
buReadAgain.Enabled:=false;
buReadAgain.caption:=kStartAutoRead;//care... this must be right...
   //see OnClick handler.
end;

procedure TTCP001f1.buReadAgainClick(Sender: TObject);
begin
if buReadAgain.caption=kStartAutoRead then begin
   tiReadAgain.Enabled:=true;
   buReadAgain.caption:='Stop auto-read';
   end //no ; here
 else begin
   tiReadAgain.Enabled:=false;
   buReadAgain.caption:=kStartAutoRead;
   end;
end;

procedure TTCP001f1.tiReadAgainTimer(Sender: TObject);
begin
SendButtonClick(nil);
end;

procedure TTCP001f1.buClearMemoClick(Sender: TObject);
begin
DisplayMemo.clear;
end;

procedure TTCP001f1.Button1Click(Sender: TObject);
begin
TCP001fAbt.show;
end;

end.

